//visual parameters
int numGridX = 6;
int numGridY = 7;
int windowWidth = 480;
int windowHeight = windowWidth;
float fretboardX = 50;
float fretboardY = 50;
float rightMargin = 150;
float noteRadius = 15;
float fretboardHeight = windowHeight - (fretboardY * 2);
float fretboardWidth = windowWidth - (fretboardX + rightMargin);
int nonChordColor = #555555;
int chordColor = #0000FF;
int rootColor = #FF0000;

// Global state
String[] scaleList = {"Major", "Pentatonic", "Harmonic Minor"};
ClickableString scale = new ClickableString(scaleList, fretboardWidth + noteRadius, 150, 20);
String[] noteList = {"C", "C#", "D", "Eb", "E", "F", "F#", "G", "G#", "A", "Bb", "B"};
ClickableString scaleRoot = new ClickableString(noteList, fretboardWidth + noteRadius, 100, 20);
String[] chordList = {"Triad", "Seventh", "Power"};
ClickableString chordType = new ClickableString(chordList, fretboardWidth + noteRadius, 200, 20);
String[] modeList = {"Ionian", "Dorian", "Phrygian", "Lydian", "Mixolydian", "Aeolian", "Locrian"};
ClickableString modeType = new ClickableString(modeList, fretboardWidth + noteRadius, 250, 20);

ClickableString[] allStrings = new ClickableString[6 + numGridX];
MoveAllGridX moveAllGridX = new MoveAllGridX("Move All Strings", fretboardWidth + noteRadius*3, GetYFor(0) - noteRadius, 20);
String[] hideList = {"Hide Nonchord Tones", "Show Nonchord Tones"};
ClickableString hideTones = new ClickableString(hideList, fretboardWidth + noteRadius, 400, 12);

int[][] scales = { {1, 0, 2, 0, 3, 4, 0, 5, 0, 6, 0, 7},   // Major
                   {1, 0, 0, 3, 0, 4, 0, 5, 0, 0, 7, 0},    // Pentatonic
                   {1, 0, 2, 3, 0, 4, 0, 5, 6, 0, 0, 7}   // Harmonic Minor
                  };
int[][] chords = {
                   {1, 3, 5},   // Triad
                   {1, 3, 5, 7}, // Seventh
                   {1, 5}       // Power
                 };
void setup() 
{
  size(windowWidth, windowWidth);
  smooth();
  textSize(18);
  
  int[] standardTune = {4, 9, 2, 7, 11};
  
  for(int i = 0; i < numGridX; ++i)
  {
    allStrings[i] = new ClickableString(noteList, GetXFor(i) - 5, GetYFor(0) - noteRadius, 20);
    allStrings[i].state = standardTune[i % standardTune.length];
  }
  
  allStrings[numGridX] = scale;
  allStrings[numGridX + 1] = scaleRoot;
  allStrings[numGridX + 2] = moveAllGridX;
  allStrings[numGridX + 3] = chordType;
  allStrings[numGridX + 4] = modeType;
  allStrings[numGridX + 5] = hideTones;
}

float GetXFor(int string)
{
  return fretboardX + string * (fretboardWidth / (numGridX + 1));
}

float GetYFor(int fret)
{
  return fretboardY + fret * (fretboardHeight / (numGridY));
}

void DrawGrid()
{
  stroke(0);
  for(int x = 0; x < numGridX; ++x)
    line(GetXFor(x), GetYFor(0), GetXFor(x), GetYFor(numGridY - 1));
  for(int y = 0; y < numGridY; ++y)
    line(GetXFor(0), GetYFor(y), GetXFor(numGridX - 1), GetYFor(y));
  
  // now for each line draw all the notes
  noStroke();
  for(int x = 0; x < numGridX; ++x)
  {
    int root = allStrings[x].state;
    for(int y = 0; y < numGridY; ++y)
    {
      int modeOffset = 0;
      for(int i = 0; i < scales[scale.state].length; i++)
        if(scales[scale.state][i] == modeType.state + 1)
          modeOffset = i;
      
      int note = (root + y - (scaleRoot.state - modeOffset) + 12) % 12;
      int scaleDegree = scales[scale.state][note];
      
      // if it's not a note, keep fishing.
      if(scaleDegree == 0)
        continue;
      
      // Offset by the current mode
      scaleDegree = (((scaleDegree - 1 - modeType.state) + modeList.length)% (modeList.length)) + 1;
      
      // Check if it's in the chord
      boolean inChord = false;  
      for(int i = 0; i< chords[chordType.state].length; ++i)
        if(chords[chordType.state][i] == scaleDegree)
          inChord = true;
      
      // Choose the color appropriately
      if(scaleDegree == 1)
        fill(rootColor, 180);
      else if (inChord)
        fill(chordColor, 180);
      else
      {
        if(hideTones.state != 0)
          continue;
        fill(nonChordColor, 180);
      }
      ellipse(GetXFor(x), GetYFor(y), 2*noteRadius, 2*noteRadius);
      fill(#FFFFFF);
      textSize(20);
      text(Integer.toString(scaleDegree), GetXFor(x), GetYFor(y) + (textAscent() + textDescent())/2);
    }
  }
}


void draw() {
  background(226);
  for(int i = 0; i < allStrings.length; ++i)
    allStrings[i].draw();
  DrawGrid();
}

void mouseClicked()
{
  for(int i = 0; i < allStrings.length; ++i)
    allStrings[i].HandleClick();
}

class ClickableString
{
  ClickableString(String[] tempStrings, float tempX, float tempY, float tempSize)
  {
    strings = tempStrings;
    x = tempX;
    y = tempY;
    size = tempSize;
  }
  
  void draw()
  {
    textSize(size);
    fill(0);
    text(GetString(), x, y);
  }
  
  void HandleClick()
  {
    textSize(size);
    // Run the good old rect gauntlet
    if((mouseX >= x) &&
       (mouseX <= x + textWidth(GetString())) &&
       (mouseY >= y - textAscent()) &&
       (mouseY <= y + textDescent()))
    {
      DoClick();
    }
  }
  
  void DoClick()
  {
      if(mouseButton == LEFT)
        state++;
      else
        state--;
      state = (state + strings.length) % strings.length;
  }
  
  String GetString()
  {
    return strings[state];
  }
  
  String[] strings;
  float x, y;
  float size;
  int state = 0;
};

class MoveAllGridX extends ClickableString
{
  MoveAllGridX(String string, float tempX, float tempY, float tempSize)
  {
    super(new String[0], tempX, tempY, tempSize);
    strings = new String[1];
    strings[0] = string;
  }
  
  void DoClick()
  {
    for(int i = 0; i < numGridX; ++i)
      allStrings[i].DoClick();
  }
};
